<?php

/**
 * @file
 * Controller routines for Juicebox XML.
 */

namespace Drupal\juicebox\Controller;

use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Controller routines for Juicebox XML.
 */
abstract class JuiceboxXmlControllerBase implements ContainerInjectionInterface {

  protected $configFactory;
  protected $settings = array();

  /**
   * Constructor
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
    // Fetch and store the Juicebox-specific global settings.
    $this->settings = $config_factory->get('juicebox.settings')->get();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Create the actual controller instance.
    return new static($container->get('config.factory'));
  }

  /**
   * Common controller for the Juicebox XML.
   *
   * @param string $args
   *   The arguments passed from the router.
   * @return Response $xml
   *   A Symfony response object containing the XML information.
   */
  public function xmlController($args = '') {
    $xml = '';
    // Try to build the XML.
    try {
      // Initialize data and test access.
      $this->init($args);
      if (!$this->access()) {
        throw new AccessDeniedHttpException();
      }
      // Get the XML.
      $xml = $this->getXml();
      // Package the XML as a Response object.
      $xml_obj = new Response($xml);
      // Set headers that apply only to XML requests.
      $xml_obj->headers->set('Content-Type', 'application/xml; charset=utf-8');
      $xml_obj->headers->set('X-Robots-Tag', 'noindex');
      if ($this->settings['enable_cors']) {
        $xml_obj->headers->set('Access-Control-Allow-Origin', '*');
      }
      return $xml_obj;
    }
    catch (Exception $e) {
      $message = 'Exception building Juicebox XML: !message in %function (line %line of %file).';
      watchdog_exception('juicebox', $e, $message);
      throw new NotFoundHttpException();
    }
  }

  /**
   * Initialize the controller based on the passed arg data.
   *
   * @param string $args
   *   The arguments passed from the router.
   */
  abstract protected function init($args);

  /**
   * Check access to the Drupal data that will be used to build the gallery.
   *
   * @return boolean
   *   Returns TRUE if access is allowed for the current user and FALSE if not.
   *   Can also return NULL if access cannot be determined.
   */
  abstract protected function access();

  /**
   * Get the Juicebox XML.
   *
   * @return string $xml
   *   The Juicebox XML for the gallery.
   */
  abstract protected function getXml();

}
